<?php
/**
* Handles the "setup".
*
* This class provides all methods neccessary to "setup" Phpdoc and check the 
* current setup.
* 
* @version 	0.2alpha
* @author		Ulf Wendel <ulf@redsys.de>
*/
class PhpdocSetupHandler extends PhpdocArgvHandler {

	/**
	* Name of the target directory.
	* @var		string	$target
	* @access	private
	*/								
	var $target = "";	
	
	/**
	* Name of the application parsed
	* @var	string	$application
	* @see	setApplication()
	*/
	var $application;
	
	/**
	* Array of packages in the application
	* @var	array	$packages
	* @see	addPackage()
	*/
	var $packages = array();
	
	/**
	* Structure of an empty package
	* @var	array	$emptyPackage
	* @see	addPackage(), package
	*/
	var	$emptyPackage = array (
															"name"									=> "",
															"empty"									=> false,
															"files"									=> array(),
															"classes"								=> array(),
															"classnamesToFilenames"	=> array(),
															"classTree"							=> array()
													);	
	
	/**
	* Basedir for all file operations
	* @var	string	$basedir
	* @see	setApplication(), setBasedir()
	*/
	var $basedir = "";
	
	/**
	* Suffix of all source code files in the application
	* If you used other file suffixes than ".php" in you have to override this
	* variable using setSourceFileSuffix()
	* @var	array	sourceFileSuffix
	* @see	setSourceFileSuffix()
	*/
	var	$sourceFileSuffix = array ( "php" );	
	
	/**
	* Sets the name of your application and the basedir for all file operations
	* @param	string	$application	name of the application
	* @param	string	$basedir			basedir for all file operations (optional)
	* @return	bool		$ok
	* @throws	wrong_param	
	* @access	public
	* @see		addPackage()
	*/	
	function setApplication($application, $basedir="") {
		if (""==$application) {
			$this->err = new exception_runtime_wrongparam("No application name given.", 10, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		$this->application = $application;
		
		if (""!=$basedir && "/"!=substr($basedir, -1)) 
			$basedir.= "/";
		$this->basedir = $basedir;
		
		return true;
	} // end func setApplication
	
	/**
	* Adds a package to the application
	* @param	string	$package			name of the package
	* @param	mixed		$directories	list of directories the package consists of (optional)
	* @param	string	$basedir			basedir used when resolving directories (optional) (default: basedir)
	* @param	boolean	$flag_subdir	include subdirectories? (default: true)
	* @return	bool		$ok
	* @throws	wrong_param
	* @access	public
	* @see		setApplication(), setPackageFiles(), addPackageFiles()
	*/
	function addPackage($package, $directories="", $basedir="", $flag_subdir = true) {
		if (""==$package) {
			$this->err = new exception_runtime_wrongparam("No Packagename specified.", 13, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		if (!is_array($directories)) 
			$directories = array($directories);
			
		if (""!=$basedir && "/"!=substr($basedir, -1))
			$basedir.="/";
		else
			$basedir = $this->basedir;
		
		$this->packages[$package] = $this->emptyPackage;
		$this->packages[$package]["name"] = $package;
		
		$fileHandler = new PhpdocFileHandler;				
		
		reset($directories);
		while (list($k, $directory)=each($directories)) {
		
			$files = $fileHandler->getFilesInDirectory($basedir.$directory, $this->sourceFileSuffix, $flag_subdir);
			reset($files);
			while (list($k, $filename)=each($files)) 
				$this->packages[$package]["files"][] = $filename;				
				
		}
		
		if (0==count($this->packages[$package]["files"]))
			$this->packages[$package]["empty"];
		
		return true;
	} // end func addPackage
	
	/**
	* Sets the files that belong to a package. 
	* Note that you override previously defined files. 
	* Use addPackageFiles() to add files.
	* @param	string	$package	name of the package
	* @param	mixed		$files		name of a single file or array of files
	* @return	bool		$ok
	* @throws	runtime_wrongparam
	* @access	public
	* @see		addPackage(), addPackageFiles()
	*/
	function setPackageFiles($package, $files) {
		if (""==$package) {
			$this->err = new exception_runtime_wrongparam("No package specified.", 17, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		if (!is_array($files))
			$files = array($files);
		
		if (!isset($this->packages[$package])) {
			$this->packages[$package] = $this->emptyPackage;
			$this->packages[$package]["name"] 	= $package;
		}

		$this->packages[$package]["files"] = $files;		
		
		if (0==count($files))
			$this->packages[$package]["empty"] = true;
		
		return true;
	} // end func setPackageFiles
	
	/**
	* Adds files to a package. Use setPackageFiles() to override previously defined files
	* @brother	setPackageFiles
	* @see			setPackageFiles(), addPackage()
	*/
	function addPackageFiles($package, $files) {
		if (""==$package) {
			$this->err = new exception_runtime_wrongparam("No package specified.", 18, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		
		if (!is_array($files))
			$files = array($files);
			
		if (!isset($this->package[$package])) {
			$this->packages[$package] = $this->emptyPackage;
			$this->packages[$package]["name"] 	= $package;
		}

		reset($files);
		while (list($k, $file)=each($files)) 
			$this->packages[$package]["files"][] = $file;

		if (0==count($this->packages[$package]["files"]))
			$this->packages[$package]["empty"];
						
		return true;
	} // end func addPackageFiles
	
	
	/**
	* Suffix of all source code files in the application
	* By default only files with the suffix ".php" are recognized as
	* php source code files and parsed. If you used other
	* suffixes such as ".inc" you have to tell phpdoc to parse
	* them.
	* @param	mixed	$suffix		string with one suffix or array of suffixes
	* @return	bool	$ok
	* @throws	runtime_wrongparam
	* @access	private
	* @see		sourceFileSuffix
	*/
	function setSourceFileSuffix($suffix) {
		if ( (!is_array($suffix) && ""==$suffix) || (is_array($suffix) && 0==count($suffix)) ) {
			$this->err = new exception_runtime_wrongparam("No suffix specified.", 15, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}
		if (!is_array($suffix)) 
			$suffix = array($suffix);		
		
		$this->sourceFileSuffix = $suffix;	
		return true;
	} // end func setSourceFileSuffix
	
	
	/**
	* Sets the target where the generated files are saved. Tries to create 
	* the directory if not existant.
	* @param	string	$target
	* @return	bool		$ok 
	* @throws runtime_wrongparam
	* @access	public
	*/
	function setTarget($target) {
		if (""==$target) {
			$this->err = new exception_runtime_wrongparam("No target specified.", 3, __FILE__, __LINE__);
			return false;
		}
		
		if (!is_dir($target)) {
			$ok = @mkdir($target, 0755);
			if (!$ok) {
				$this->err = new exception_io("setTarget(), can't create a directory '$target'.", 4, __FILE__, __LINE__);
				return false;
			}
		}
			
		$this->target = $target;
		return true;
	} // end func setTarget

	/**
	* Checks the current status of the object. Are all necessary informations 
	* to start parsing available?
	* @param	array		$errors
	* @access	private
	* @return	array		$errors
	*/
	function checkStatus($errors="") {
		if (!is_array($errors))
			$errors = array();
/*
				
		if (0==count($this->files) && ""==$this->directory) 
			$errors[] = array (
													"msg" 	=> "No source files or source directory specified.",
													"type"	=> "misconfiguration",
													"errno"	=> 6
												);
												
		if (0!=count($this->files) && ""!=$this->directory) 
			$errors[] = array(
													"msg"		=> "Define eighter some files or a diretory.",
													"type"	=> "misconfiguration",
													"errno"	=> 7
											);
	*/	
		return $errors;
	} // end func checkStatus

} // end class PhpdocSetupHandler
?>